/**
 * Check for Endpoint.
 * @param {any} object to be checked
 * @return {boolean} true if object is an Endpoint
 */
export function isEndpoint(object: any): boolean;
/**
 * Instanciate interceptors from its definitions.
 * @param {Interceptor[]|string[]|undefined} interceptors
 * @param {Object} owner
 * @return {Interceptor[]}
 */
export function instanciateInterceptors(interceptors: Interceptor[] | string[] | undefined, owner: any): Interceptor[];
/**
 * @typedef {Object} ConnectionState
 */
/**
 * Connection endpoint.
 */
export class Endpoint {
    /**
     * @param {string} name endpoint name
     * @param {Object} owner of the endpoint (service)
     * @param {Object} options
     * @param {function(Endpoint,Endpoint):void} [options.didConnect] called after receiver is present
     * @param {Interceptor[]|string[]} [options.interceptors] interceptors
     */
    constructor(name: string, owner: any, options: {
        didConnect?: (arg0: Endpoint, arg1: Endpoint) => void;
        interceptors?: Interceptor[] | string[];
    });
    name: string;
    owner: any;
    /**
     *
     * @param {Endpoint} endpoint
     * @param {Endpoint} other
     */
    didConnect(endpoint: Endpoint, other: Endpoint): void;
    interceptors: Interceptor[];
    /**
     * Name as presented for humans.
     * @return {string}
     */
    get displayName(): string;
    /**
     * Indicate whatever we are a default endpoint.
     * Default means buildin.
     * @return {boolean} false
     */
    get isDefault(): boolean;
    /**
     * Indicate whatever we are a dummy endpoint.
     * Dummy endpoints are used duiring construction of the endpoint mesh.
     * @return {boolean} false
     */
    get isDummy(): boolean;
    /**
     * Mapping of properties used in toString.
     * @return {Object}
     */
    get toStringAttributes(): any;
    /**
     *
     * @param {Object} options
     */
    connectionNamesWithStates(options?: any): string[];
    toString(): string;
    get identifier(): string;
    /**
     * @return {boolean} false
     */
    get isIn(): boolean;
    /**
     * @return {boolean} false
     */
    get isOut(): boolean;
    /**
     * Deliver data flow direction.
     * @return {string|undefined} delivers data flow direction 'in', 'out', 'inout' or undefined
     */
    get direction(): string | undefined;
    toJSON(): {
        in: boolean;
        out: boolean;
        open: boolean;
        dummy: boolean;
        connected: string | string[];
        interceptors: any[];
    };
    /**
     * Additional attributes to present in json output.
     */
    get jsonAttributes(): any[];
    toJSONWithOptions(options: any): {
        in: boolean;
        out: boolean;
        open: boolean;
        dummy: boolean;
        connected: string | string[];
        interceptors: any[];
    };
    /**
     * @return {boolean} true if there is at least one interceptor assigned
     */
    get hasInterceptors(): boolean;
    get isOpen(): boolean;
    /**
     * Can we form a connection to the other side.
     * in to out and out to in.
     * @param {Endpoint} other
     * @return {boolean} true if we can be connected to the other endpoint
     */
    connectable(other: Endpoint): boolean;
    /**
     * @return {boolean} true if there is at least one connection
     */
    get hasConnections(): boolean;
    /**
     * Are we connected to a endpoint.
     * @param {Endpoint} other to check for the connection
     * @return {boolean} true if there is a connection to the other endpoint
     */
    isConnected(other: Endpoint): boolean;
    /**
     * Actually start with the communication.
     * @param {Endpoint} other
     * @param {boolean} [backpointer] true if this is the call from back call from the other side
     */
    openConnection(other: Endpoint, backpointer?: boolean): void;
    /**
     * Actually stop the communication.
     * @param {Endpoint} other
     * @param {boolean} [backpointer] true if this is the call from back call from the other side
     */
    closeConnection(other: Endpoint, backpointer?: boolean): void;
    /**
     * Opens all connections.
     */
    openConnections(): void;
    /**
     * Closes all connections.
     */
    closeConnections(): void;
    /**
     * @return {Iterable<Endpoint>}
     */
    connections(): Iterable<Endpoint>;
    /**
     *
     * @param {Endpoint} connection
     * @param {boolean} [backpointer] true if this is the call form back call from the other side
     */
    addConnection(connection: Endpoint, backpointer?: boolean): void;
    /**
     * Actually stop the communication.
     * @param {Endpoint} other
     * @param {boolean} [backpointer] true if this is the call form back call from the other side
     */
    removeConnection(other: Endpoint, backpointer?: boolean): void;
    /**
     * Deliver state for a given connection.
     * @param {Endpoint} other
     * @return {ConnectionState}
     */
    getConnectionState(other: Endpoint): ConnectionState;
    /**
     * Set state for a given connection.
     * @param {Endpoint} other
     * @param {ConnectionState} state
     */
    setConnectionState(other: Endpoint, state: ConnectionState): void;
    get receivingInterceptors(): any[];
}
export type ConnectionState = any;
import { Interceptor } from "@kronos-integration/interceptor";
